const http = require('http');
const https = require('https');
require('http-shutdown').extend();
const fs = require("fs");
const mkdirp = require('mkdirp');
const JSONStream = require('JSONStream');
const evS = require('event-stream');
const ndjson = require('ndjson');


const FileInfo = require('./FileInfo');

var server = null;
var cfg = null;
var cb_msg = null;

function json2NdJSON(ss) {
	var lfname = [ss.action, ss.SN, 'DAU', ss.dau_idx, ss.type, ss.idx].join('_') + '.ndjson';
	var fname = FileInfo.getOrSetFilePath() + '/' + lfname;
	var resp = this.resp;
	var sname = ['DAU', ss.dau_idx, ss.type, ss.idx].join(' ');
	if (!fs.existsSync(fname)) FileInfo.addFile(lfname);

	// Pipe the ndjson serialized output to the file-system.
	var toNdJSonStream = ndjson.stringify();

	var out = toNdJSonStream.pipe(fs.createWriteStream(fname, {
		flags: 'a',
		encoding: 'utf8'
	}));
	ss.frames.forEach(function (ss_data) {
		//		infoMsg(chalk.green(JSON.stringify(ss_data)));
		toNdJSonStream.write(ss_data);
	});
	toNdJSonStream.end();
	out.on('finish', function () {
		FileInfo.incFilesValues(lfname, {
			nb_items_recv: ss.frames.length
		});
		FileInfo.save();

		infoMsg('Stored %d items from %s', ss.frames.length, sname);
		resp.writeHead(200, "OK", {
			'Content-Type': 'text/plain'
		});
		resp.end();
	});
}

function processHttpStreamPost(req, resp) {
	const fpath = [cfg.path, req.headers['user-agent'], req.headers['x-spcu-action'], req.headers['x-spcu-version']].join('/');

	mkdirp.sync(fpath);
	FileInfo.getOrSetFilePath(fpath);
	FileInfo.save();

	var ss = {
		action: null,
		SN: null,
		dau_idx: null,
		type: null,
		idx: null
	};

	var jsonStream = JSONStream.parse('dau_array.*');
	req.pipe(jsonStream).pipe(evS.mapSync(json2NdJSON.bind({
		resp: resp
	})));

}

/* Note: Data structure specs.
 * data = dau_array : [{ 
 * 	action: 'insertmeasure',
 * 	SN: '',
 * 	dau_idx:,
 * 	idx:
 * 	type: ('OPTIC' || 'PT' || 'EXTERNAL'),
 * 	frames : [{time:'', raw:, comp:, trigger:, }, ...]
 * }, ...]
 */

function configServer(req, resp) {
	//infoMsg('headers:' + JSON.stringify(req.headers));
	infoMsg("Receiving datas...");

	req.on('connect', (err) => {
		infoMsg('Receiving datas starting ....');
	});
	req.on('error', (err) => {
		infoMsg(err);
		resp.statusCode = 400;
		resp.end();
	});
	req.on('end', (err) => {
		//infoMsg('End of transaction');
	});

	if (req.method == 'POST') {
		processHttpStreamPost(req, resp);
	} else {
		resp.writeHead(404, "NOT FOUND", {
			'Content-Type': 'text/plain'
		});
		//resp.end();
	}

}

function infoMsg() {
	arguments[0] = "EDAS &rArr; DataCollector: " + arguments[0];
	if (cb_msg) cb_msg(arguments);
}

function loadHttpConfig() {
	var opts = null;

	if (cfg.spcu.proto == 'https') {
		opts = {
			key: [fs.readFileSync(cfg.spcu.key, {
				encoding: 'utf-8'
			})],
			cert: [fs.readFileSync(cfg.spcu.cert, {
				encoding: 'utf-8'
			})],
			ca: [fs.readFileSync(cfg.spcu.ca, {
				encoding: 'utf-8'
			})],
			requestCert: true,
			rejectUnauthorized: false,
		};
		server = https.createServer(opts, configServer);
	} else {
		server = http.createServer(configServer);
	}
}

function startServer(_cfg, _cb_msg) {
	cfg = _cfg;
	cb_msg = _cb_msg;
	if (!startServer.first) {
		startServer.first = 1;
		process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0";
	}

	infoMsg("Listening on port %d", cfg.spcu.port);


	loadHttpConfig();

	server.withShutdown();
	server.listen(cfg.spcu.port);
}

function stopServer() {
	if (!server) return;
	server.shutdown(function () {
		infoMsg('Server stopped');
	});
}

function initServer() {


}
//sensorStream.sendToSensorStream(cfg);

module.exports = {
	startServer: startServer,
	stopServer: stopServer
}
