const FileInfo = require('./FileInfo');
const fs = require("fs");
const http = require('http');
const https = require('https');
const Readable = require('stream').Readable;
const util = require('util');
const path = require('path');

var sensorStream = null;
var cb_msg = null;
var cfg = null;
var stopServer = false;
var console = null;
var document = null;
var conn = {
    req_stack: [],
};
var sync_time = {
    start: null,
    end: null
};
var wait_delay_hdle = null;
var msg = '';

function infoMsg() {
    arguments[0] = "DataCollector &rArr; Server: " + arguments[0];
    if (cb_msg) cb_msg(arguments);
}

function infoConsole() {
    if (console) console.log.call(arguments);
}

function getFd(fname) {
    if (!getFd.fds) getFd.fds = {};
    if (!getFd.fds[fname]) {
        getFd.fds[fname] = fs.openSync(fname, 'a+');
    }
    return getFd.fds[fname];
}

function removeFd(fname) {
    if (getFd.fds[fname]) {
        fs.closeSync(getFd.fds[fname]);
        getFd.fds[fname] = null;
    }
}

function removeAllFd() {
    if (getFd.fs) {
        FileInfo.getFiles().forEach(function (lfname) {
            var fname = FileInfo.getOrSetFilePath() + '/' + lfname;
            removeFd(fname);
        });
    }
}

function readLineFd(fd, offset) {
    var line = '';
    var c = new Buffer('0');
    var pos = 0;
    var nb_bytes = 0;
    const end_char = '\n'.charCodeAt(0);
    const bend_char = '\r'.charCodeAt(0);
    for (var pos = 0; c[0] != end_char; pos++) {
        var d_nb_bytes = fs.readSync(fd, c, 0, 1, offset++);
        if (!d_nb_bytes) break;
        if (c[0] != end_char && c[0] != bend_char) line += c.toString();
        nb_bytes += d_nb_bytes;
    }

    return {
        size: nb_bytes,
        line: line
    };
}

function prepareHttpHeader(filePath) {

    var http_info = filePath.split('/');
    return {
        'user-agent': http_info[1],
        'x-spcu-action': http_info[2],
        'x-spcu-version': http_info[3],
    };
}

function prepareJSONObject(fname) {
    var ss_info = path.basename(fname).split('.')[0].split('_');
    return {
        action: ss_info[0],
        SN: ss_info[1],
        dau_idx: parseInt(ss_info[3]),
        type: ss_info[4],
        idx: parseInt(ss_info[5]),
    };
}

function prepareMessage() {
    var http_hd = null;
    var nb_ss = 0;
    conn.req_stack = [];
    console.log("send_data starts");
    var tot_bytes = 0;
    var msg = '';

    msg += '{"dau_array": [';
    FileInfo.getFiles().forEach(function (lfname) {
        var fname = FileInfo.getOrSetFilePath() + '/' + lfname;
        var ss_colon = (nb_ss > 0) ? ',' : '';
        var fd = getFd(fname);
        var offset = FileInfo.getValue(lfname, 'offset');
        var nb_bytes = 0;
        var ss = prepareJSONObject(lfname);
        var sname = [ss.dau_idx, ss.type, ss.idx].join(' ');
        //infoMsg('Reading %s datas', sname);
        msg += util.format('%s{"action":"%s","SN":"%s","dau_idx":%i,"type":"%s","idx":%i,"frames":[',
            ss_colon, ss.action, ss.SN, ss.dau_idx, ss.type, ss.idx);
        var nb_sensor_sent = 0;
        for (var i = 0; i < cfg.cloud.nb_data_per_frame; i++) {
            var res = readLineFd(fd, offset + nb_bytes);
            var colon = (i > 0) ? "," : "";
            //infoMsg('readLine returns', res);

            nb_bytes += res.size;
            if (res.size) {
                msg += colon + res.line;
                nb_sensor_sent++;
            } else {
                infoMsg("No more datas to send from %s", sname);
                removeFd(fname);
                //FileInfo.removeFile(fname);
                break;
            }
        }
        if (nb_sensor_sent > 0) {
            console.log('Sending %d', nb_sensor_sent);
            conn.req_stack.push({
                lfname: lfname,
                nb_items_sent: nb_sensor_sent,
                offset: nb_bytes,
                ss: {
                    SN: ss.SN,
                    dau_idx: ss.dau_idx,
                    stype: ss.type,
                    sid: ss.idx
                },
            });
        }
        msg += ']}';
        nb_ss++;

    });
    msg += ']}';
    return msg;
}

function sendData(post_req) {
    try {
        post_req.write(msg);
        post_req.end();

    } catch (err) {
        (e) =>
        infoMsg(err.stack || err);
    }
    return true;
}

function lSetTimeout(cb, delay) {
    if (cb) {
        lSetTimeout.hdle = setTimeout(cb, delay);
        return lSetTimeout.hdle;
    } else {
        if (lSetTimeout.hdle) {
            clearTimeout(lSetTimeout.hdle);
            lSetTimeout.hdle = null;
        }
    }
}

function prepareRemote(http_info) {
    var http_client = cfg.cloud.proto == 'https' ? https : http;
    msg = prepareMessage();
    var http_opts = {
        host: cfg.cloud.hostname,
        port: cfg.cloud.port,
        path: cfg.cloud.rel_path,
        json: true,
        method: 'POST',
        headers: {
            'content-type': 'application/json; charset=UTF-8',
            'user-agent': http_info['user-agent'],
            'x-spcu-action': http_info['x-spcu-action'],
            'x-spcu-version': http_info['x-spcu-version'],
            'Content-Length': msg.length,
            'Expect': '100-continue',
        }
    };
    /*
        if (cfg.cloud.do_chunk) {
            http_opts.headers['Transfer-Encoding'] = 'chunked';
        }
    */
    if (cfg.cloud.proto == 'https') {
        http_opts.pfx = fs.readFileSync(cfg.cloud.pfx);
        http_opts.passphrase = cfg.cloud.pfx_pass;
        http_opts.requestCert = true;
        http_opts.rejectUnauthorized = false;
    }

    return http_client.request(http_opts, function (res) {
        res.setEncoding('utf8');
        res.on('data', () => {
            infoMsg('Data sent');
        });
        res.on('end', () => {
            infoMsg('data sent OK.');
            conn.req_stack.forEach(function (item) {
                infoMsg('DAU_%d_%s_%d sents %d sensor datas', item.ss.dau_idx, item.ss.stype, item.ss.sid, item.nb_items_sent);
                console.log('Updating %s', item.lfname);
                FileInfo.incFilesValues(item.lfname, {
                    nb_items_sent: item.nb_items_sent,
                    offset: item.offset,
                });
            });
            FileInfo.save(function () {
                var delay = (cfg.cloud.frequency * 1000) - (performance.now() - sync_time.start);
                if (delay > 0) {
                    infoMsg("Waiting %dms before sending", parseInt(delay));
                    lSetTimeout(startSend, delay);
                    return;
                } else {
                    startSend();
                }
            });
        });
        res.on('error', (e) => {
            infoMsg('req error', JSON.stringify(e));
            sync_time.end = performance.now();
            lSetTimeout(startSend, 2000);
        });
    });
}

function startSend() {
    if (stopServer) {
        infoMsg('Stop by user !!!!');
        if (startSend.post_req) startSend.post_req.abort();
        return;
    }
    if (FileInfo.checkAllSent()) {
        var ev = new Event('sensors_all_sent');
        document.dispatchEvent(ev);
        infoMsg('All datas had been sent.');
        removeAllFd();
        return;
    }
    sync_time.start = performance.now();
    var http_info = prepareHttpHeader(FileInfo.getOrSetFilePath());
    startSend.post_req = prepareRemote(http_info);
    startSend.post_req.on('error', function (err) {
        infoMsg("Error: %s &rArr; Can't connect to %s://%s/%s:%s", err.code, cfg.cloud.proto, cfg.cloud.hostname, cfg.cloud.path, cfg.cloud.port);
        lSetTimeout(startSend, 2000);
    });
    startSend.post_req.on('connect', function () {
        infoMsg("Connected: %s", JSON.stringify(err));

    });
    sendData(startSend.post_req);
}

function sendToSensorStream(_cfg, _cb_msg, _doc, _console) {
    if (_cb_msg) cb_msg = _cb_msg;
    if (_cfg) cfg = _cfg;
    if (!document) document = _doc;
    if (_console) {
        console = _console;
        console.log('Console initialized !!!');
    }
    stopServer = false;
    startSend();
}

function stopSent() {
    if (stopServer) return;
    lSetTimeout();
    removeAllFd();
    stopServer = true;
    infoMsg('Stop sending datas to cloud.');
}


module.exports = {
    sendToSensorStream: sendToSensorStream,
    stopSent: stopSent
};
